""" Wrapper to include the main library modules """
from .core import ULog
from . import px4
from . import _version

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



#! /usr/bin/env python
"""
Extract the raw gps communication from an ULog file.
"""

from __future__ import print_function

import argparse
import os
import sys

from .core import ULog

#pylint: disable=too-many-locals, unused-wildcard-import, wildcard-import

def main():
    """
    Command line interface
    """
    parser = argparse.ArgumentParser(
        description='Extract the raw gps communication from an ULog file')
    parser.add_argument('filename', metavar='file.ulg', help='ULog input file')

    def is_valid_directory(parser, arg):
        """Check if valid directory"""
        if not os.path.isdir(arg):
            parser.error('The directory {} does not exist'.format(arg))
        # File exists so return the directory
        return arg
    parser.add_argument('-o', '--output', dest='output', action='store',
                        help='Output directory (default is CWD)',
                        metavar='DIR', type=lambda x: is_valid_directory(parser, x))
    parser.add_argument('-x', '--ignore', dest='ignore', action='store_true',
                        help='Ignore string parsing exceptions', default=False)
    parser.add_argument('-i', '--instance', dest='required_instance', action='store',
                        help='GPS instance. Use 0 (default)'
                        + 'for main GPS, 1 for secondary GPS reciever.',
                        default=0)

    args = parser.parse_args()
    ulog_file_name = args.filename
    disable_str_exceptions = args.ignore
    required_instance = int(args.required_instance)

    msg_filter = ['gps_dump']
    ulog = ULog(ulog_file_name, msg_filter, disable_str_exceptions)
    data = ulog.data_list

    output_file_prefix = os.path.basename(ulog_file_name)
    # strip '.ulg'
    if output_file_prefix.lower().endswith('.ulg'):
        output_file_prefix = output_file_prefix[:-4]

    # write to different output path?
    if args.output is not None:
        output_file_prefix = os.path.join(args.output, output_file_prefix)

    to_dev_filename = output_file_prefix + '_' + str(required_instance) + '_to_device.dat'
    from_dev_filename = output_file_prefix + '_' + str(required_instance) + '_from_device.dat'


    if len(data) == 0:
        print("File {0} does not contain gps_dump messages!".format(ulog_file_name))
        sys.exit(0)

    gps_dump_data = data[0]

    # message format check
    field_names = [f.field_name for f in gps_dump_data.field_data]
    if not 'len' in field_names or not 'data[0]' in field_names:
        print('Error: gps_dump message has wrong format')
        sys.exit(-1)

    if len(ulog.dropouts) > 0:
        print("Warning: file contains {0} dropouts".format(len(ulog.dropouts)))

    print("Creating files {0} and {1}".format(to_dev_filename, from_dev_filename))

    with open(to_dev_filename, 'wb') as to_dev_file:
        with open(from_dev_filename, 'wb') as from_dev_file:
            msg_lens = gps_dump_data.data['len']
            instances = gps_dump_data.data.get('instance', [0]*len(msg_lens))
            for i in range(len(gps_dump_data.data['timestamp'])):
                instance = instances[i]
                msg_len = msg_lens[i]
                if instance == required_instance:
                    if msg_len & (1<<7):
                        msg_len = msg_len & ~(1<<7)
                        file_handle = to_dev_file
                    else:
                        file_handle = from_dev_file
                    for k in range(msg_len):
                        file_handle.write(gps_dump_data.data['data['+str(k)+']'][i])



"""
PX4-specific ULog helper
"""
from __future__ import print_function
import numpy as np

__author__ = "Beat Kueng"


class PX4ULog(object):
    """
    This class contains PX4-specific ULog things (field names, etc.)
    """

    def __init__(self, ulog_object):
        """
        @param ulog_object: ULog instance
        """
        self._ulog = ulog_object

    def get_mav_type(self):
        """ return the MAV type as string from initial parameters """

        mav_type = self._ulog.initial_parameters.get('MAV_TYPE', None)
        return {0: 'Generic',
                1: 'Fixed Wing',
                2: 'Quadrotor',
                3: 'Coaxial helicopter',
                4: 'Normal helicopter with tail rotor',
                5: 'Ground installation',
                6: 'Ground Control Station',
                7: 'Airship, controlled',
                8: 'Free balloon, uncontrolled',
                9: 'Rocket',
                10: 'Ground Rover',
                11: 'Surface Vessel, Boat, Ship',
                12: 'Submarine',
                13: 'Hexarotor',
                14: 'Octorotor',
                15: 'Tricopter',
                16: 'Flapping wing',
                17: 'Kite',
                18: 'Onboard Companion Controller',
                19: 'Two-rotor VTOL (Tailsitter)',
                20: 'Quad-rotor VTOL (Tailsitter)',
                21: 'Tiltrotor VTOL',
                22: 'VTOL Standard', #VTOL reserved 2
                23: 'VTOL reserved 3',
                24: 'VTOL reserved 4',
                25: 'VTOL reserved 5',
                26: 'Onboard Gimbal',
                27: 'Onboard ADSB Peripheral'}.get(mav_type, 'unknown type')

    def get_estimator(self):
        """return the configured estimator as string from initial parameters"""

        mav_type = self._ulog.initial_parameters.get('MAV_TYPE', None)
        if mav_type == 1: # fixed wing always uses EKF2
            return 'EKF2'

        mc_est_group = self._ulog.initial_parameters.get('SYS_MC_EST_GROUP', 2)
        return {0: 'INAV',
                1: 'LPE',
                2: 'EKF2',
                3: 'Q'}.get(mc_est_group, 'unknown ({})'.format(mc_est_group))


    def add_roll_pitch_yaw(self):
        """ convenience method to add the fields 'roll', 'pitch', 'yaw' to the
        loaded data using the quaternion fields (does not update field_data).

        Messages are: 'vehicle_attitude.q' and 'vehicle_attitude_setpoint.q_d',
        'vehicle_attitude_groundtruth.q' and 'vehicle_vision_attitude.q' """

        self._add_roll_pitch_yaw_to_message('vehicle_attitude')
        self._add_roll_pitch_yaw_to_message('vehicle_vision_attitude')
        self._add_roll_pitch_yaw_to_message('vehicle_attitude_groundtruth')
        self._add_roll_pitch_yaw_to_message('vehicle_attitude_setpoint', '_d')


    def _add_roll_pitch_yaw_to_message(self, message_name, field_name_suffix=''):

        message_data_all = [elem for elem in self._ulog.data_list if elem.name == message_name]
        for message_data in message_data_all:
            q = [message_data.data['q'+field_name_suffix+'['+str(i)+']'] for i in range(4)]
            roll = np.arctan2(2.0 * (q[0] * q[1] + q[2] * q[3]),
                              1.0 - 2.0 * (q[1] * q[1] + q[2] * q[2]))
            pitch = np.arcsin(2.0 * (q[0] * q[2] - q[3] * q[1]))
            yaw = np.arctan2(2.0 * (q[0] * q[3] + q[1] * q[2]),
                             1.0 - 2.0 * (q[2] * q[2] + q[3] * q[3]))
            message_data.data['roll'+field_name_suffix] = roll
            message_data.data['pitch'+field_name_suffix] = pitch
            message_data.data['yaw'+field_name_suffix] = yaw


    def get_configured_rc_input_names(self, channel):
        """
        find all RC mappings to a given channel and return their names

        :param channel: input channel (0=first)
        :return: list of strings or None
        """
        ret_val = []
        for key in self._ulog.initial_parameters:
            param_val = self._ulog.initial_parameters[key]
            if key.startswith('RC_MAP_') and param_val == channel + 1:
                ret_val.append(key[7:].capitalize())

        if len(ret_val) > 0:
            return ret_val
        return None



#! /usr/bin/env python
"""
Extract and compare parameters from two ULog files
"""

from __future__ import print_function

import argparse
import sys
import re
from html import escape

from .core import ULog
#pylint: disable=unused-variable, too-many-branches

def get_defaults(ulog, default):
    """ get default params from ulog """
    assert ulog.has_default_parameters, "Log does not contain default parameters"

    if default == 'system': return ulog.get_default_parameters(0)
    if default == 'current_setup': return ulog.get_default_parameters(1)
    raise Exception('invalid value \'{}\' for --default'.format(default))

def main():
    """Commande line interface"""
    parser = argparse.ArgumentParser(description='Extract parameters from an ULog file')
    parser.add_argument('filename1', metavar='file1.ulg', help='ULog input file')
    parser.add_argument('filename2', metavar='file2.ulg', help='ULog input file')

    parser.add_argument('-l', '--delimiter', dest='delimiter', action='store',
                        help='Use delimiter in CSV (default is \',\')', default=',')

    parser.add_argument('-i', '--initial', dest='initial', action='store_true',
                        help='Only extract initial parameters. (octave|csv)', default=False)

    parser.add_argument('-t', '--timestamps', dest='timestamps', action='store_true',
                        help='Extract changed parameters with timestamps. (csv)', default=False)

    parser.add_argument('-f', '--format', dest='format', action='store', type=str,
                        help='csv|octave|qgc', default='csv')

    parser.add_argument('output_filename', metavar='params.txt',
                        type=argparse.FileType('w'), nargs='?',
                        help='Output filename (default=stdout)', default=sys.stdout)

    parser.add_argument('--ignore', dest='ignore', action='store_true',
                        help='Ignore string parsing exceptions', default=False)

    parser.add_argument('-d', '--default', dest='default', action='store', type=str,
                        help='Select default param values instead of configured '
                        'values (implies --initial). Valid values: system|current_setup',
                        default=None)

    args = parser.parse_args()
    ulog_file_name1 = args.filename1
    ulog_file_name2 = args.filename2
    disable_str_exceptions = args.ignore

    message_filter = []
    if not args.initial: message_filter = None

    ulog1 = ULog(ulog_file_name1, message_filter, disable_str_exceptions)
    ulog2 = ULog(ulog_file_name2, message_filter, disable_str_exceptions)

    params1 = ulog1.initial_parameters
    params2 = ulog2.initial_parameters
    changed_params1 = ulog1.changed_parameters
    changed_params2 = ulog2.changed_parameters
    if args.default is not None:
        params1 = get_defaults(ulog1, args.default)
        args.initial = True

    changed_param1_list = dict([(i[1],i[2]) for i in changed_params1])
    param_keys1 = sorted(params1.keys())
    param_keys1_minusinflight = sorted(params1.keys()) - changed_param1_list.keys()
    param_keys2 = sorted(params2.keys())
    delimiter = args.delimiter
    output_file = args.output_filename
    version1 = ''
    version2 = ''
    output_file.write('\n')

    if 'boot_console_output' in ulog1.msg_info_multiple_dict:
        console_output = ulog1.msg_info_multiple_dict['boot_console_output'][0]
        escape(''.join(console_output))
        version = re.search('Build datetime:',str(console_output))
        version1 = str(console_output)[version.end():version.start()+36]

    if 'boot_console_output' in ulog2.msg_info_multiple_dict:
        console_output = ulog2.msg_info_multiple_dict['boot_console_output'][0]
        escape(''.join(console_output))
        version = re.search('Build datetime:',str(console_output))
        version2 = str(console_output)[version.end():version.start()+36]

    if version1 != version2:
        output_file.write('\n')
        output_file.write('New Firmware \n')
        output_file.write('Build:')
        output_file.write(version1)
        output_file.write('\n')
        output_file.write('↓')
        output_file.write('\n')
        output_file.write('Build:')
        output_file.write(version2)
        output_file.write('\n\n')

    if args.format == "csv":
        if (set(param_keys2) - set(param_keys1)) or (set(param_keys1) - set(param_keys2)):
            for param_key2 in set(param_keys2) - set(param_keys1):
                output_file.write('New: ')
                output_file.write(param_key2)
                output_file.write(', ')
                output_file.write(str(round(params2[param_key2],6)))
                output_file.write('\n')
            for param_key1 in set(param_keys1) - set(param_keys2):
                output_file.write('Deleted: ')
                output_file.write(param_key1)
                output_file.write('\n')
                output_file.write('\n')
        output_file.write('\nChanged pre-flight:\n')
        for param_key1 in param_keys1_minusinflight:
            for param_key2 in param_keys2:
                if (param_key1 == param_key2) and (param_key1 != 'LND_FLIGHT_T_LO') and (param_key1 != 'LND_FLIGHT_T_HI') and (param_key1 != 'COM_FLIGHT_UUID'):
                    if isinstance(params1[param_key1],float):
                        if (abs(params1[param_key1] - params2[param_key2]) > 0.001):
                                output_file.write(param_key1)
                                output_file.write(': ')
                                output_file.write(str(round(params1[param_key1],5)))
                                output_file.write(' -> ')
                                output_file.write(str(round(params2[param_key2],5)))
                                output_file.write('\n')
                    else:
                        if (params1[param_key1] != params2[param_key2]):
                                output_file.write(param_key1)
                                output_file.write(': ')
                                output_file.write(str(params1[param_key1]))
                                output_file.write(' -> ')
                                output_file.write(str(params2[param_key2]))
                                output_file.write('\n')
        last_matched = ''
        last_matched_val = 0.0
        output_file.write('\nChanged in-flight:\n')
        for param_key2 in param_keys2:
            for i in changed_params2:
                if (param_key2 == i[1]) and (param_key2 != 'LND_FLIGHT_T_LO') and (param_key1 != 'LND_FLIGHT_T_HI') and (param_key2 != 'COM_FLIGHT_UUID'):
                    if isinstance(params2[param_key2],float):
                        if (abs(params2[param_key2] - i[2]) > 0.001):
                            if (last_matched != i[1]):
                                output_file.write(param_key2)
                                output_file.write(': ')
                                output_file.write(str(round(params2[param_key2],5)))
                                output_file.write(' -> ')
                                output_file.write(str(round(i[2],5)))
                                output_file.write('\n')
                                last_matched = i[1]
                                last_matched_val = i[2]
                            else:
                                output_file.write(last_matched)
                                output_file.write(': ')
                                output_file.write(str(round(last_matched_val,5)))
                                output_file.write(' -> ')
                                output_file.write(str(round(i[2],5)))
                                output_file.write('\n')
                    else:
                        if (params2[param_key2] != i[2]):
                            if (last_matched != i[1]):
                                output_file.write(param_key2)
                                output_file.write(': ')
                                output_file.write(str(params2[param_key2]))
                                output_file.write(' -> ')
                                output_file.write(str(i[2]))
                                output_file.write('\n')
                            else:
                                output_file.write(last_matched)
                                output_file.write(': ')
                                output_file.write(str(last_matched_val,5))
                                output_file.write(' -> ')
                                output_file.write(str(i[2],5))
                                output_file.write('\n')


#! /usr/bin/env python
"""
Display logged messages from an ULog file
"""

from __future__ import print_function

import argparse

from .core import ULog
#pylint: disable=invalid-name

def main():
    """Commande line interface"""

    parser = argparse.ArgumentParser(description='Display logged messages from an ULog file')
    parser.add_argument('filename', metavar='file.ulg', help='ULog input file')
    parser.add_argument('-i', '--ignore', dest='ignore', action='store_true',
                        help='Ignore string parsing exceptions', default=False)

    args = parser.parse_args()
    ulog_file_name = args.filename
    disable_str_exceptions = args.ignore

    msg_filter = [] # we don't need the data messages
    ulog = ULog(ulog_file_name, msg_filter, disable_str_exceptions)


    for m in ulog.logged_messages:
        m1, s1 = divmod(int(m.timestamp/1e6), 60)
        h1, m1 = divmod(m1, 60)
        print("{:d}:{:02d}:{:02d} {:}: {:}".format(
            h1, m1, s1, m.log_level_str(), m.message))






#!/usr/bin/env python
"""Python log parser for ULog.

This module allows you to parse ULog files, which are used within the PX4
autopilot middleware.

The file format is documented on https://dev.px4.io/advanced-ulog-file-format.html

"""

from __future__ import print_function
import os
import sys
import versioneer

from setuptools import setup, find_packages

DOCLINES = __doc__.split("\n")

CLASSIFIERS = """\
Development Status :: 1 - Planning
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Other
Topic :: Software Development
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Scientific/Engineering :: Mathematics
Topic :: Scientific/Engineering :: Physics
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

# pylint: disable=invalid-name

setup(
    name='pyulog_Selab',
    maintainer="Takenori Hashimoto",
    maintainer_email="takenori.hashimoto@selab.jp",
    description=DOCLINES[0],
    long_description="\n".join(DOCLINES[2:]),
    url='https://github.com/takhashimoton/SEL-pyulog.git',
    author='Beat Kueng',
    author_email='beat-kueng@gmx.net',
    download_url='https://github.com/takhashimoton/SEL-pyulog.git',
    license='BSD 3-Clause',
    classifiers=[_f for _f in CLASSIFIERS.split('\n') if _f],
    platforms=["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
    install_requires=['numpy'],
    tests_require=['nose', 'ddt'],
    test_suite='nose.collector',
    entry_points = {
        'console_scripts': [
            'ulog_extract_gps_dump=pyulog.extract_gps_dump:main',
            'ulog_info=pyulog.info:main',
            'ulog_messages=pyulog.messages:main',
            'ulog_params=pyulog.params:main',
            'ulog2csv=pyulog.ulog2csv:main',
            'ulog2kml=pyulog.ulog2kml:main',
        ],
    },
    packages=find_packages(),
    version='1.0.3',
    cmdclass=versioneer.get_cmdclass(),
)
